@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 
-from debian.deb822 import Deb822
+from debian.deb822 import Deb822, Release
+import email.utils
 import os
 import sys
 import shutil
@@ -14,6 +15,7 @@ have_qemu = os.getenv("HAVE_QEMU", "yes") == "yes"
 have_unshare = os.getenv("HAVE_UNSHARE", "yes") == "yes"
 have_binfmt = os.getenv("HAVE_BINFMT", "yes") == "yes"
 run_ma_same_tests = os.getenv("RUN_MA_SAME_TESTS", "yes") == "yes"
+cmd = os.getenv("CMD", "./mmdebstrap")
 
 default_dist = os.getenv("DEFAULT_DIST", "unstable")
 all_dists = ["oldstable", "stable", "testing", "unstable"]
@@ -33,11 +35,21 @@ all_variants = [
 default_format = "auto"
 all_formats = ["auto", "directory", "tar", "squashfs", "ext2", "null"]
 
-only_dists = []
-
 mirror = os.getenv("mirror", "http://127.0.0.1/debian")
 hostarch = subprocess.check_output(["dpkg", "--print-architecture"]).decode().strip()
 
+release_path = f"./shared/cache/debian/dists/{default_dist}/Release"
+if not os.path.exists(release_path):
+    print("path doesn't exist:", release_path, file=sys.stderr)
+    print("run ./make_mirror.sh first", file=sys.stderr)
+    exit(1)
+if os.getenv("SOURCE_DATE_EPOCH") is not None:
+    s_d_e = os.getenv("SOURCE_DATE_EPOCH")
+else:
+    with open(release_path) as f:
+        rel = Release(f)
+    s_d_e = str(email.utils.mktime_tz(email.utils.parsedate_tz(rel["Date"])))
+
 separator = (
     "------------------------------------------------------------------------------"
 )
@@ -75,6 +87,7 @@ def main():
         help="exit after first num failures or errors.",
     )
     parser.add_argument("--mode", metavar="mode", help="only run tests with this mode")
+    parser.add_argument("--dist", metavar="dist", help="only run tests with this dist")
     args = parser.parse_args()
 
     # copy over files from git or as distributed
@@ -105,16 +118,10 @@ def main():
             "/usr/share/mmdebstrap/hooks", "shared/hooks", dirs_exist_ok=True
         )
 
-    onlyrun = None
-    if len(sys.argv) > 1:
-        onlyrun = sys.argv[1]
-
     tests = []
     with open("coverage.txt") as f:
         for test in Deb822.iter_paragraphs(f):
             name = test["Test"]
-            if args.test and name not in args.test:
-                continue
             dists = test.get("Dists", default_dist)
             if dists == "any":
                 dists = all_dists
@@ -144,11 +151,7 @@ def main():
             else:
                 formats = formats.split()
             for dist in dists:
-                if only_dists and dist not in only_dists:
-                    continue
                 for mode in modes:
-                    if args.mode and args.mode != mode:
-                        continue
                     for variant in variants:
                         for fmt in formats:
                             skipreason = skip(
@@ -172,27 +175,60 @@ def main():
                                 tt = "null"
                             tests.append((tt, name, dist, mode, variant, fmt))
 
+    torun = []
+    num_tests = len(tests)
+    if args.test:
+        # check if all given tests are either a valid name or a valid number
+        for test in args.test:
+            if test in [name for (_, name, _, _, _, _) in tests]:
+                continue
+            if not test.isdigit():
+                print(f"cannot find test named {test}", file=sys.stderr)
+                exit(1)
+            if int(test) >= len(tests) or int(test) <= 0 or str(int(test)) != test:
+                print(f"test number {test} doesn't exist", file=sys.stderr)
+                exit(1)
+
+        for i, (_, name, _, _, _, _) in enumerate(tests):
+            # if either the number or test name matches, then we use this test,
+            # otherwise we skip it
+            if name in args.test:
+                torun.append(i)
+            if str(i + 1) in args.test:
+                torun.append(i)
+        num_tests = len(torun)
+
     starttime = time.time()
     skipped = defaultdict(list)
     failed = []
     num_success = 0
+    num_finished = 0
     for i, (test, name, dist, mode, variant, fmt) in enumerate(tests):
+        if torun and i not in torun:
+            continue
         print(separator, file=sys.stderr)
         print("(%d/%d) %s" % (i + 1, len(tests), name), file=sys.stderr)
         print("dist: %s" % dist, file=sys.stderr)
         print("mode: %s" % mode, file=sys.stderr)
         print("variant: %s" % variant, file=sys.stderr)
         print("format: %s" % fmt, file=sys.stderr)
-        if i > 0:
+        if num_finished > 0:
             currenttime = time.time()
             timeleft = timedelta(
-                seconds=int((len(tests) - i) * (currenttime - starttime) / i)
+                seconds=int(
+                    (num_tests - num_finished)
+                    * (currenttime - starttime)
+                    / num_finished
+                )
             )
             print("time left: %s" % timeleft, file=sys.stderr)
+        if failed:
+            print("failed: %d" % len(failed))
+        num_finished += 1
         with open("tests/" + name) as fin, open("shared/test.sh", "w") as fout:
             for line in fin:
-                for e in ["CMD", "SOURCE_DATE_EPOCH"]:
-                    line = line.replace("{{ " + e + " }}", os.getenv(e))
+                line = line.replace("{{ CMD }}", cmd)
+                line = line.replace("{{ SOURCE_DATE_EPOCH }}", s_d_e)
                 line = line.replace("{{ DIST }}", dist)
                 line = line.replace("{{ MIRROR }}", mirror)
                 line = line.replace("{{ MODE }}", mode)
@@ -215,11 +251,18 @@ def main():
                 print(f"skipped because of {reason}", file=sys.stderr)
                 continue
         print(separator, file=sys.stderr)
+        if args.dist and args.dist != dist:
+            print(f"skipping because of --dist={args.dist}", file=sys.stderr)
+            continue
+        if args.mode and args.mode != mode:
+            print(f"skipping because of --mode={args.mode}", file=sys.stderr)
+            continue
         proc = subprocess.Popen(argv)
         try:
             proc.wait()
         except KeyboardInterrupt:
-            proc.kill()
+            proc.terminate()
+            proc.wait()
             break
         print(separator, file=sys.stderr)
         if proc.returncode != 0:
@@ -237,7 +280,7 @@ def main():
         file=sys.stderr,
     )
     if skipped:
-        print("skipped %d:" % len(skipped), file=sys.stderr)
+        print("skipped %d:" % sum([len(v) for v in skipped.values()]), file=sys.stderr)
         for reason, l in skipped.items():
             print(f"skipped because of {reason}:", file=sys.stderr)
             for t in l:
